#version 330 core

#if FOG_TYPE != 0
uniform sampler3D fogTex;
layout(std140) uniform Environment {
	vec4 ambienceAndEmission;
	vec4 envi0ColorAndFactor;
	vec4 envi1Color;
	vec4 fogColor;
	mat4 fogMatrix;
	mat4 viewInv;
};
#endif

#ifndef SHADOW_MAP
uniform mat4 projection;
#endif
uniform mat4 modelview;
 
in vec4 vertexPosition;
in vec3 vertexNormal;
in vec3 vertexTangent;
in vec4 vertexTexCoord0;
in vec4 vertexTexCoord1;
in vec4 vertexTexCoord2;
in vec4 vertexTexCoord3;
in vec4 vertexColor;

out vec4 position;
out vec3 normal;
out vec3 tangent;
out vec3 bitangent;
out vec4 texCoord0;
out vec4 texCoord1;
out vec4 texCoord2;
out vec4 texCoord3;
out vec4 color;
#if FOG_TYPE != 0
flat out float eyeFog;
#endif
 
void main(void) 
{
	position = modelview * vertexPosition;
#ifdef SHADOW_MAP
	gl_Position = position;
#else
	gl_Position = projection * position;
#endif
	
	//not a proper transformation if modelview_matrix involves non-uniform scaling
	mat3 normalMatrix = mat3(modelview);
	normal = normalize( normalMatrix * vertexNormal );
	tangent = normalize( normalMatrix * vertexTangent );
	bitangent = cross(tangent, normal);
	
	texCoord0 = vertexTexCoord0;
	texCoord1 = vertexTexCoord1;
	texCoord2 = vertexTexCoord2;
	texCoord3 = vertexTexCoord3;
	
	color = vertexColor;
	
#if FOG_TYPE != 0
	vec3 fogCoord = fogMatrix[3].xyz; //Separate statement due to Intel bug
	eyeFog = texture(fogTex, fogCoord).r;
#endif
}